import { GCOptimizedList } from "encompass-gc-optimized-collections";
import { Component } from "./component";
import { ComponentManager } from "./component_manager";
import { Entity } from "./entity";
import { EntityCreator } from "./entity_creator";
import { EntityManager } from "./entity_manager";
import { Message } from "./message";
import { MessageManager } from "./message_manager";
import { ComponentMessage } from "./types/component_message";
import { EntityMessage } from "./types/entity_message";
import { Type } from "./types/generic_type";
/**
 * An Engine is the Encompass notion of an ECS System. Engines are responsible
 * for reading the game state and emitting messages, as well as mutating
 * Entities and Components.
 *
 * A particular kind of Component may be modified by only one Engine. If
 * you have two Engines which mutate the same Component type, an error
 * will be thrown.
 *
 * An Engine which Reads a particular Message type is guaranteed to run
 * after all Engines which Emit that particular Message type.
 */
export declare abstract class Engine {
    readonly emit_message_types: GCOptimizedList<Type<Message>>;
    readonly read_message_types: GCOptimizedList<Type<Message>>;
    readonly mutate_component_types: GCOptimizedList<Type<Component>>;
    private entity_manager;
    private entity_creator;
    private component_manager;
    private message_manager;
    constructor(entity_manager: EntityManager, entity_creator: EntityCreator, component_manager: ComponentManager, message_manager: MessageManager);
    abstract update(dt: number): void;
    /**
     * Returns a newly instantiated Entity.
     */
    protected create_entity(): Entity;
    /**
     * Emits a newly instantiated Message of MessageType.
     * The message can and should be modified after this method is called.
     *
     * @param MessageType
     * @throws When the MessageType is undeclared.
     */
    protected emit_message<TMessage extends Message>(MessageType: Type<TMessage>): TMessage;
    /**
     * A convenience method for emitting messages that have a component property.
     * The message can and should be modified after this method is called.
     *
     * @param MessageType
     * @param component
     */
    protected emit_component_message<TMessage extends ComponentMessage>(MessageType: Type<TMessage>, component: Component): TMessage;
    /**
     * A convenience method for emitting messages that have an entity property.
     * The message can and should be modified after this method is called.
     *
     * @param MessageType
     * @param entity
     */
    protected emit_entity_message<TMessage extends EntityMessage>(MessageType: Type<TMessage>, entity: Entity): TMessage;
    /**
     * Returns the Entity instance with the given ID.
     *
     * @param entity_id
     */
    protected get_entity(entity_id: number): Entity | undefined;
    /**
     * Gets all active components of ComponentType.
     * The components are marked as readonly. For mutable components,
     * use get_mutable_components_by_type.
     *
     * @param ComponentType
     */
    protected read_components<TComponent extends Component>(ComponentType: Type<TComponent>): GCOptimizedList<Readonly<TComponent>>;
    /**
     * A convenience method for the case of singleton Components.
     *
     * @param ComponentType
     * @throws If more than one component of ComponentType exists
     * @returns A component of type ComponentType
     */
    protected read_component<TComponent extends Component>(ComponentType: Type<TComponent>): Readonly<TComponent> | null;
    /**
     * Gets all active components of ComponentType.
     * The components are mutable, but the ComponentType must
     * be declared in mutates_component_types on the Engine
     * or an error will be thrown.
     *
     * @param ComponentType
     * @throws When the ComponentType is not declared.
     */
    protected read_components_mutable<TComponent extends Component>(ComponentType: Type<TComponent>): GCOptimizedList<TComponent>;
    /**
     * A convenience method for the case of singleton Components.
     *
     * @param ComponentType
     * @throws When the ComponentType is not declared.
     * @throws When there is more than one Component.
     * @returns A component of type ComponentType
     *
     */
    protected read_component_mutable<TComponent extends Component>(ComponentType: Type<TComponent>): TComponent | null;
    /**
     * Makes a given component instance mutable.
     *
     * @param component
     * @throws When the component is not declared in mutate_component_types.
     */
    protected make_mutable<TComponent extends Component>(component: Readonly<TComponent>): TComponent;
    /**
     * Gets all inactive components of ComponentType.
     * The components are marked as readonly.
     *
     * @param ComponentType
     */
    protected read_inactive_components<TComponent extends Component>(ComponentType: Type<TComponent>): GCOptimizedList<Readonly<TComponent>>;
    /**
     * Gets all messages of MessageType.
     *
     * @param MessageType
     * @throws When the message type is undeclared.
     */
    protected read_messages<TMessage extends Message>(MessageType: Type<TMessage>): GCOptimizedList<Readonly<TMessage>>;
    /**
     * Returns true if any messages of given type were emitted this frame.
     *
     * @param MessageType
     * @throws When the message type is undeclared.
     */
    protected some<TMessage extends Message>(MessageType: Type<TMessage>): boolean;
}
