"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
const emit_undeclared_message_1 = require("./errors/emit_undeclared_message");
const illegal_component_mutation_1 = require("./errors/illegal_component_mutation");
const read_undeclared_message_1 = require("./errors/read_undeclared_message");
const singleton_read_1 = require("./errors/singleton_read");
/**
 * An Engine is the Encompass notion of an ECS System. Engines are responsible
 * for reading the game state and emitting messages, as well as mutating
 * Entities and Components.
 *
 * A particular kind of Component may be modified by only one Engine. If
 * you have two Engines which mutate the same Component type, an error
 * will be thrown.
 *
 * An Engine which Reads a particular Message type is guaranteed to run
 * after all Engines which Emit that particular Message type.
 */
class Engine {
    constructor(entity_manager, entity_creator, component_manager, message_manager) {
        this.emit_message_types = new encompass_gc_optimized_collections_1.GCOptimizedList();
        this.read_message_types = new encompass_gc_optimized_collections_1.GCOptimizedList();
        this.mutate_component_types = new encompass_gc_optimized_collections_1.GCOptimizedList();
        this.entity_manager = entity_manager;
        this.entity_creator = entity_creator;
        this.component_manager = component_manager;
        this.message_manager = message_manager;
    }
    /**
     * Returns a newly instantiated Entity.
     */
    create_entity() {
        return this.entity_creator.create_entity();
    }
    /**
     * Emits a newly instantiated Message of MessageType.
     * The message can and should be modified after this method is called.
     *
     * @param MessageType
     * @throws When the MessageType is undeclared.
     */
    emit_message(MessageType) {
        if (this.emit_message_types.hasValue(MessageType)) {
            return this.message_manager.create_message(MessageType);
        }
        else {
            const message_error = new emit_undeclared_message_1.EmitUndeclaredMessageError(this.constructor.name, MessageType);
            throw message_error.toString();
        }
    }
    /**
     * A convenience method for emitting messages that have a component property.
     * The message can and should be modified after this method is called.
     *
     * @param MessageType
     * @param component
     */
    emit_component_message(MessageType, component) {
        const message = this.emit_message(MessageType);
        message.component = component;
        return message;
    }
    /**
     * A convenience method for emitting messages that have an entity property.
     * The message can and should be modified after this method is called.
     *
     * @param MessageType
     * @param entity
     */
    emit_entity_message(MessageType, entity) {
        const message = this.emit_message(MessageType);
        message.entity = entity;
        return message;
    }
    /**
     * Returns the Entity instance with the given ID.
     *
     * @param entity_id
     */
    get_entity(entity_id) {
        return this.entity_manager.get_entity(entity_id);
    }
    /**
     * Gets all active components of ComponentType.
     * The components are marked as readonly. For mutable components,
     * use get_mutable_components_by_type.
     *
     * @param ComponentType
     */
    read_components(ComponentType) {
        return this.component_manager.get_components_by_component_type(ComponentType);
    }
    /**
     * A convenience method for the case of singleton Components.
     *
     * @param ComponentType
     * @throws If more than one component of ComponentType exists
     * @returns A component of type ComponentType
     */
    read_component(ComponentType) {
        const components = this.component_manager.get_components_by_component_type(ComponentType);
        if (components.size === 0) {
            return null;
        }
        else if (components.size === 1) {
            return components.get(0);
        }
        else {
            const multiple_component_error = new singleton_read_1.SingletonReadError(this.constructor.name, ComponentType);
            throw multiple_component_error.toString();
        }
    }
    /**
     * Gets all active components of ComponentType.
     * The components are mutable, but the ComponentType must
     * be declared in mutates_component_types on the Engine
     * or an error will be thrown.
     *
     * @param ComponentType
     * @throws When the ComponentType is not declared.
     */
    read_components_mutable(ComponentType) {
        if (this.mutate_component_types.hasValue(ComponentType)) {
            return this.component_manager.get_components_by_component_type(ComponentType);
        }
        else {
            const mutate_error = new illegal_component_mutation_1.IllegalComponentMutationError(this.constructor.name, ComponentType);
            throw mutate_error.toString();
        }
    }
    /**
     * A convenience method for the case of singleton Components.
     *
     * @param ComponentType
     * @throws When the ComponentType is not declared.
     * @throws When there is more than one Component.
     * @returns A component of type ComponentType
     *
     */
    read_component_mutable(ComponentType) {
        if (this.mutate_component_types.hasValue(ComponentType)) {
            const components = this.component_manager.get_components_by_component_type(ComponentType);
            if (components.size === 0) {
                return null;
            }
            else if (components.size === 1) {
                return components.get(0);
            }
            else {
                const multiple_component_error = new singleton_read_1.SingletonReadError(this.constructor.name, ComponentType);
                throw multiple_component_error.toString();
            }
        }
        else {
            const mutate_error = new illegal_component_mutation_1.IllegalComponentMutationError(this.constructor.name, ComponentType);
            throw mutate_error.toString();
        }
    }
    /**
     * Makes a given component instance mutable.
     *
     * @param component
     * @throws When the component is not declared in mutate_component_types.
     */
    make_mutable(component) {
        if (this.mutate_component_types.hasValue(component.type)) {
            return component;
        }
        else {
            const mutate_error = new illegal_component_mutation_1.IllegalComponentMutationError(this.constructor.name, component.type);
            throw mutate_error.toString();
        }
    }
    /**
     * Gets all inactive components of ComponentType.
     * The components are marked as readonly.
     *
     * @param ComponentType
     */
    read_inactive_components(ComponentType) {
        return this.component_manager.get_inactive_components_by_component_type(ComponentType);
    }
    /**
     * Gets all messages of MessageType.
     *
     * @param MessageType
     * @throws When the message type is undeclared.
     */
    read_messages(MessageType) {
        if (this.read_message_types.hasValue(MessageType)) {
            return this.message_manager.get_messages_by_message_type(MessageType);
        }
        else {
            const message_error = new read_undeclared_message_1.ReadUndeclaredMessage(this.constructor.name, MessageType);
            throw message_error.toString();
        }
    }
    /**
     * Returns true if any messages of given type were emitted this frame.
     *
     * @param MessageType
     * @throws When the message type is undeclared.
     */
    some(MessageType) {
        if (this.read_message_types.hasValue(MessageType)) {
            return this.message_manager.get_messages_by_message_type(MessageType).size > 0;
        }
        else {
            const message_error = new read_undeclared_message_1.ReadUndeclaredMessage(this.constructor.name, MessageType);
            throw message_error.toString();
        }
    }
}
exports.Engine = Engine;
