"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
const engine_1 = require("../../engine");
const illegal_component_mutation_1 = require("../../errors/illegal_component_mutation");
/**
 * A ComponentModifier is an Engine which is responsible
 * for consuming ComponentMessages and modifying the referenced
 * Component in response.
 *
 * ComponentModifiers are defined by their message_type
 * and the modify function they implement.
 *
 * It is an anti-pattern to read Entities or Components
 * within a ComponentModifier.
 */
class ComponentModifier extends engine_1.Engine {
    constructor() {
        super(...arguments);
        this.component_to_messages = new encompass_gc_optimized_collections_1.GCOptimizedMap();
    }
    update(dt) {
        this.retrieve_and_map_messages_by_component_instance();
        for (const [component, messages] of this.component_to_messages.entries()) {
            const entity = this.get_entity(component.entity_id);
            if (entity !== undefined && messages.size > 0) {
                if (this.mutate_component_types.hasValue(component.type)) {
                    this.modify(component, messages, dt);
                }
                else {
                    const mutation_error = new illegal_component_mutation_1.IllegalComponentMutationError(this.constructor.name, component.type);
                    throw mutation_error.toString();
                }
            }
        }
        this.clean();
    }
    clean() {
        for (const [component, messages] of this.component_to_messages.entries()) {
            if (!this.get_entity(component.entity_id)) {
                this.component_to_messages.delete(component);
            }
            messages.clear();
        }
    }
    retrieve_and_map_messages_by_component_instance() {
        const messages = this.read_messages(this.read_message_types.get(0));
        for (const message of messages.values()) {
            if (!this.component_to_messages.has(message.component)) {
                this.component_to_messages.set(message.component, new encompass_gc_optimized_collections_1.GCOptimizedList());
            }
            this.component_to_messages.get(message.component).add(message);
        }
    }
}
exports.ComponentModifier = ComponentModifier;
