"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
const engine_1 = require("../../engine");
/**
 * An EntityModifier is an Engine which is responsible
 * for consuming EntityMessages and modifying Entities
 * in response.
 *
 * EntityModifiers are defined by their message_type
 * and the modify function they implement.
 *
 * It is an anti-pattern to read data from Entities
 * or Components inside an EntityModifier.
 */
class EntityModifier extends engine_1.Engine {
    constructor() {
        super(...arguments);
        this.entity_id_to_messages = new encompass_gc_optimized_collections_1.GCOptimizedMap();
    }
    update(dt) {
        this.retrieve_and_map_messages_by_entity_id();
        for (const [entity_id, entity_messages] of this.entity_id_to_messages.entries()) {
            const entity = this.get_entity(entity_id);
            if (entity !== undefined && entity_messages.size > 0) {
                this.modify(entity, entity_messages, dt);
            }
        }
        this.clean();
    }
    clean() {
        for (const [entity_id, messages] of this.entity_id_to_messages.entries()) {
            if (!this.get_entity(entity_id)) {
                this.entity_id_to_messages.delete(entity_id);
            }
            messages.clear();
        }
    }
    retrieve_and_map_messages_by_entity_id() {
        const messages = this.read_messages(this.read_message_types.get(0));
        for (const [_, message] of messages.entries()) {
            if (!this.entity_id_to_messages.has(message.entity.id)) {
                this.entity_id_to_messages.set(message.entity.id, new encompass_gc_optimized_collections_1.GCOptimizedList());
            }
            this.entity_id_to_messages.get(message.entity.id).add(message);
        }
    }
}
exports.EntityModifier = EntityModifier;
