import { GCOptimizedList, GCOptimizedSet } from "encompass-gc-optimized-collections";
import { Component } from "./component";
import { ComponentManager } from "./component_manager";
import { DrawLayerManager } from "./draw_layer_manager";
import { ComponentUserDataOnly } from "./interfaces/component_user_data_only";
import { IDrawable } from "./interfaces/drawable";
import { Type } from "./types/generic_type";
/**
 * An Entity is composed of a unique internal ID
 * and a collection of Components.
 *
 * Entities do not have any implicit properties or
 * behavior. They are granted these by their collection
 * of Components.
 *
 * There is no limit to the amount of Components an
 * Entity can have, and Entities can have any number
 * of Components of a particular type.
 *
 * Entities are active by default and can be deactivated.
 * When deactivated, Entities are still tracked by
 * Engines, but the Engines temporarily ignore them.
 *
 * Note that the activation status of an Entity
 * does not affect the activation status of its Components.
 *
 * Defining logic on an Entity is an anti-pattern.
 */
export declare class Entity {
    readonly id: number;
    private _component_types;
    private component_manager;
    private component_type_to_component_bag;
    private component_type_to_active_components;
    private dirty_entities_added;
    private dirty_entities_removed;
    private entities_marked_for_destroy;
    private draw_layer_manager;
    readonly component_types: GCOptimizedSet<Type<Component>>;
    constructor(id: number, dirty_entities_added: GCOptimizedList<Entity>, dirty_entities_removed: GCOptimizedList<Entity>, entities_marked_for_destroy: GCOptimizedList<Entity>, draw_layer_manager: DrawLayerManager<IDrawable>, component_manager: ComponentManager);
    /**
     * Instantiates a Component of the given subtype
     * and adds it to the Entity.
     *
     * @param ComponentType {Type<TComponent>}
     * @param properties {ComponentUserDataOnly<TComponent>}
     */
    add_component<TComponent extends Component>(ComponentType: Type<TComponent>, properties?: ComponentUserDataOnly<TComponent>): TComponent;
    /**
     * This is a convenience method for the common case
     * of having a single component of a certain type on
     * an Entity.
     *
     * NOTE: Only returns an active Component.
     *
     * NOTE: Can return undefined if no Components of the given type exist.
     *
     * @param constructor
     * @returns An arbitrary readonly component of given type, or null if none exist.
     */
    get_component<TComponent extends Component>(constructor: Type<TComponent>): Readonly<TComponent>;
    /**
     * Gets all the active Components of a given type that exist
     * on the Entity.
     *
     * @param ComponentType
     * @returns A GCOptimizedList containing readonly references to each
     * component of the given Component type, or an empty GCOptimizedList if none exist.
     */
    get_components<TComponent extends Component>(ComponentType: Type<TComponent>): GCOptimizedList<Readonly<TComponent>>;
    /**
     * Gets all the Components of a given type that exist on the Entity,
     * including inactive Components.
     *
     * @param ComponentType
     * @returns A GCOptimizedList containing references to each
     * component of the given Component type, or an empty GCOptimizedList if none exist.
     */
    get_components_including_inactive<TComponent extends Component>(ComponentType: Type<TComponent>): GCOptimizedList<Readonly<TComponent>>;
    /**
     * Checks if the Entity has a particular active Component of ComponentType.
     *
     * @param ComponentType
     * @returns True if the given Component type is on the Entity, otherwise false.
     */
    has_component<TComponent extends Component>(ComponentType: Type<TComponent>): boolean;
    /**
     * Checks if the Entity has a particular Component of ComponentType, including inactive components.
     *
     * @param ComponentType
     * @returns True if the given Component type is on the Entity, otherwise false.
     */
    has_component_including_inactive<TComponent extends Component>(ComponentType: Type<TComponent>): boolean;
    /**
     * Removes a specific Component instance from
     * the Entity.
     *
     * @param component
     */
    remove_component(component: Component): void;
    remove_all_components(): void;
    /**
     * Activates the component, making it tracked by engines.
     * Does nothing if the component is already active.
     *
     * NOTE: Components are active by default when added to an entity.
     *
     * @param component The component to activate.
     */
    activate_component(component: Component): void;
    /**
     * Deactivates the component, making it untracked by engines.
     * Does nothing if the component is already inactive.
     *
     * @param component The component to deactivate.
     */
    deactivate_component(component: Component): void;
    /**
     * Marks the Entity to be destroyed. Note
     * that the Entity will exist until the
     * end of the World update.
     */
    destroy(): void;
}
