"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
// TODO: refactor this stuff onto component and entity manager
/**
 * An Entity is composed of a unique internal ID
 * and a collection of Components.
 *
 * Entities do not have any implicit properties or
 * behavior. They are granted these by their collection
 * of Components.
 *
 * There is no limit to the amount of Components an
 * Entity can have, and Entities can have any number
 * of Components of a particular type.
 *
 * Entities are active by default and can be deactivated.
 * When deactivated, Entities are still tracked by
 * Engines, but the Engines temporarily ignore them.
 *
 * Note that the activation status of an Entity
 * does not affect the activation status of its Components.
 *
 * Defining logic on an Entity is an anti-pattern.
 */
class Entity {
    constructor(id, dirty_entities_added, dirty_entities_removed, entities_marked_for_destroy, draw_layer_manager, component_manager) {
        this._component_types = new encompass_gc_optimized_collections_1.GCOptimizedSet();
        this.id = id;
        this.dirty_entities_added = dirty_entities_added;
        this.dirty_entities_removed = dirty_entities_removed;
        this.entities_marked_for_destroy = entities_marked_for_destroy;
        this.draw_layer_manager = draw_layer_manager;
        this.component_manager = component_manager;
        this.component_type_to_component_bag = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        this.component_type_to_active_components = new encompass_gc_optimized_collections_1.GCOptimizedMap();
    }
    get component_types() { return this._component_types; }
    /**
     * Instantiates a Component of the given subtype
     * and adds it to the Entity.
     *
     * @param ComponentType {Type<TComponent>}
     * @param properties {ComponentUserDataOnly<TComponent>}
     */
    add_component(ComponentType, properties) {
        const component = this.component_manager.create_component(ComponentType, this.id, this.draw_layer_manager, properties);
        if (!this.component_type_to_component_bag.has(ComponentType)) {
            const component_bag = new encompass_gc_optimized_collections_1.GCOptimizedList();
            const active_component_bag = new encompass_gc_optimized_collections_1.GCOptimizedList();
            this.component_type_to_component_bag.set(ComponentType, component_bag);
            this.component_type_to_active_components.set(ComponentType, active_component_bag);
            component_bag.add(component);
            active_component_bag.add(component);
        }
        else {
            this.component_type_to_component_bag.get(ComponentType).add(component);
            this.component_type_to_active_components.get(ComponentType).add(component);
        }
        this._component_types.add(ComponentType);
        this.dirty_entities_added.add(this);
        return component;
    }
    /**
     * This is a convenience method for the common case
     * of having a single component of a certain type on
     * an Entity.
     *
     * NOTE: Only returns an active Component.
     *
     * NOTE: Can return undefined if no Components of the given type exist.
     *
     * @param constructor
     * @returns An arbitrary readonly component of given type, or null if none exist.
     */
    get_component(constructor) {
        return this.get_components(constructor).get(0);
    }
    /**
     * Gets all the active Components of a given type that exist
     * on the Entity.
     *
     * @param ComponentType
     * @returns A GCOptimizedList containing readonly references to each
     * component of the given Component type, or an empty GCOptimizedList if none exist.
     */
    get_components(ComponentType) {
        if (this.component_type_to_active_components.has(ComponentType)) {
            return this.component_type_to_active_components.get(ComponentType);
        }
        else {
            return encompass_gc_optimized_collections_1.GCOptimizedList.Empty;
        }
    }
    /**
     * Gets all the Components of a given type that exist on the Entity,
     * including inactive Components.
     *
     * @param ComponentType
     * @returns A GCOptimizedList containing references to each
     * component of the given Component type, or an empty GCOptimizedList if none exist.
     */
    get_components_including_inactive(ComponentType) {
        if (this.component_type_to_component_bag.has(ComponentType)) {
            return this.component_type_to_component_bag.get(ComponentType);
        }
        else {
            return encompass_gc_optimized_collections_1.GCOptimizedList.Empty;
        }
    }
    /**
     * Checks if the Entity has a particular active Component of ComponentType.
     *
     * @param ComponentType
     * @returns True if the given Component type is on the Entity, otherwise false.
     */
    has_component(ComponentType) {
        return this.component_type_to_active_components.has(ComponentType) &&
            !this.component_type_to_active_components.get(ComponentType).empty();
    }
    /**
     * Checks if the Entity has a particular Component of ComponentType, including inactive components.
     *
     * @param ComponentType
     * @returns True if the given Component type is on the Entity, otherwise false.
     */
    has_component_including_inactive(ComponentType) {
        return this.component_type_to_component_bag.has(ComponentType) &&
            !this.component_type_to_component_bag.get(ComponentType).empty();
    }
    /**
     * Removes a specific Component instance from
     * the Entity.
     *
     * @param component
     */
    remove_component(component) {
        const component_bag = this.component_type_to_component_bag.get(component.type);
        const active_component_bag = this.component_type_to_active_components.get(component.type);
        component_bag.deleteValue(component);
        active_component_bag.deleteValue(component);
        this.dirty_entities_removed.add(this);
        this.component_manager.mark_for_removal(component);
        component.destroy();
        if (component_bag.empty()) {
            this._component_types.delete(component.type);
        }
    }
    remove_all_components() {
        for (const [type, component_list] of this.component_type_to_component_bag.entries()) {
            for (const component of component_list.values()) {
                this.component_manager.mark_for_removal(component);
                component.destroy();
            }
            component_list.clear();
        }
        for (const [type, component_list] of this.component_type_to_active_components.entries()) {
            component_list.clear();
        }
        this._component_types.clear();
        this.dirty_entities_removed.add(this);
    }
    /**
     * Activates the component, making it tracked by engines.
     * Does nothing if the component is already active.
     *
     * NOTE: Components are active by default when added to an entity.
     *
     * @param component The component to activate.
     */
    activate_component(component) {
        if (!component.active) {
            component.activate();
            this.component_type_to_active_components.get(component.type).add(component);
            this.dirty_entities_added.add(this);
            this.component_manager.mark_for_activation(component);
        }
    }
    /**
     * Deactivates the component, making it untracked by engines.
     * Does nothing if the component is already inactive.
     *
     * @param component The component to deactivate.
     */
    deactivate_component(component) {
        if (component.active) {
            component.deactivate();
            const active_component_bag = this.component_type_to_active_components.get(component.type);
            const index = active_component_bag.indexOf(component);
            if (index !== null) {
                active_component_bag.delete(index);
            }
            this.dirty_entities_removed.add(this);
            this.component_manager.mark_for_deactivation(component);
        }
    }
    /**
     * Marks the Entity to be destroyed. Note
     * that the Entity will exist until the
     * end of the World update.
     */
    destroy() {
        this.entities_marked_for_destroy.add(this);
    }
}
exports.Entity = Entity;
