"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
class EntityManager {
    constructor(entities, id_to_entity, entities_with_added_components, entities_with_removed_components, entities_marked_for_destroy) {
        this.entity_to_entity_trackers_map = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        this.component_type_to_entity_trackers_map = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        this.entities = entities;
        this.id_to_entity = id_to_entity;
        this.entities_with_added_components = entities_with_added_components;
        this.entities_with_removed_components = entities_with_removed_components;
        this.entities_marked_for_destroy = entities_marked_for_destroy;
    }
    check_entities_with_added_components() {
        for (const entity of this.entities_with_added_components.values()) {
            this.check_and_register_entity(entity);
        }
        this.entities_with_added_components.clear();
    }
    check_entities_with_removed_components() {
        for (const entity of this.entities_with_removed_components.values()) {
            if (this.entity_to_entity_trackers_map.has(entity)) {
                for (const engine of this.entity_to_entity_trackers_map.get(entity).entries()) {
                    if (this.is_entity_tracker(engine)) {
                        engine.check_and_untrack_entity(entity.id);
                    }
                }
            }
        }
        this.entities_with_removed_components.clear();
    }
    register_entity_tracker(entity_tracker) {
        for (const component_type of entity_tracker.component_types) {
            const entity_tracker_set = this.get_or_create_entity_tracker_set(component_type);
            entity_tracker_set.add(entity_tracker);
        }
    }
    register_drawable_entity_tracker(drawable_entity_tracker) {
        this.register_entity_tracker(drawable_entity_tracker);
        const entity_tracker_set = this.get_or_create_entity_tracker_set(drawable_entity_tracker.draw_component_type);
        entity_tracker_set.add(drawable_entity_tracker);
    }
    destroy_marked_entities() {
        for (const entity of this.entities_marked_for_destroy.values()) {
            entity.remove_all_components();
            if (this.entity_to_entity_trackers_map.has(entity)) {
                for (const tracker of this.entity_to_entity_trackers_map.get(entity).entries()) {
                    tracker.check_and_untrack_entity(entity.id);
                }
            }
            this.entity_to_entity_trackers_map.delete(entity);
            this.entities.deleteValue(entity);
            this.id_to_entity.delete(entity.id);
        }
        this.entities_marked_for_destroy.clear();
    }
    get_entity(id) {
        return this.id_to_entity.get(id);
    }
    is_entity_tracker(engine) {
        return engine.
            check_and_untrack_entity !== undefined;
    }
    get_or_create_entity_tracker_set(component_type) {
        if (!this.component_type_to_entity_trackers_map.has(component_type)) {
            this.component_type_to_entity_trackers_map.set(component_type, new encompass_gc_optimized_collections_1.GCOptimizedList());
        }
        return this.component_type_to_entity_trackers_map.get(component_type);
    }
    check_and_register_entity(entity) {
        for (const component_type of entity.component_types.entries()) {
            if (this.component_type_to_entity_trackers_map.has(component_type)) {
                for (const entity_tracker of this.component_type_to_entity_trackers_map.get(component_type).values()) {
                    if (entity_tracker.check_and_track_entity(entity.id)) {
                        if (!this.entity_to_entity_trackers_map.has(entity)) {
                            this.entity_to_entity_trackers_map.set(entity, new encompass_gc_optimized_collections_1.GCOptimizedSet());
                        }
                        this.entity_to_entity_trackers_map.get(entity).add(entity_tracker);
                    }
                }
            }
        }
    }
}
exports.EntityManager = EntityManager;
