"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graph_1 = require("./graph");
const priority_queue_1 = require("./priority_queue");
var SearchSymbol;
(function (SearchSymbol) {
    SearchSymbol[SearchSymbol["start"] = 0] = "start";
    SearchSymbol[SearchSymbol["finish"] = 1] = "finish";
})(SearchSymbol || (SearchSymbol = {}));
class DirectedGraph extends graph_1.Graph {
    add_edge(u, v) {
        if (this.vertex_exists(u) && this.vertex_exists(v)) {
            this._edges.get(u).push(v);
        }
    }
    add_edges(...edges) {
        for (const edge of edges) {
            this.add_edge(edge[0], edge[1]);
        }
    }
    node_dfs() {
        const discovered = new Set();
        let time = 0;
        const output = new Map();
        for (const vertex of this.vertices) {
            output.set(vertex, new Map());
        }
        const dfs_helper = (u) => {
            discovered.add(u);
            time += 1;
            output.get(u).set(SearchSymbol.start, time);
            for (const neighbor of this.neighbors(u)) {
                if (!discovered.has(neighbor)) {
                    dfs_helper(neighbor);
                }
            }
            time += 1;
            output.get(u).set(SearchSymbol.finish, time);
        };
        for (const vertex of this.vertices) {
            if (!discovered.has(vertex)) {
                dfs_helper(vertex);
            }
        }
        return output;
    }
    topological_sort() {
        const dfs = this.node_dfs();
        const priority_queue = new priority_queue_1.PriorityQueue();
        for (const [key, value] of dfs) {
            priority_queue.add(key, value.get(SearchSymbol.finish));
        }
        return priority_queue.to_array().reverse();
    }
    clone() {
        const clone = new DirectedGraph();
        clone.add_vertices(...this.vertices);
        for (const v of this.vertices) {
            for (const n of this.neighbors(v)) {
                clone.add_edge(v, n);
            }
        }
        return clone;
    }
    cyclic() {
        const sccs = this.strongly_connected_components();
        for (const scc of sccs) {
            if (scc.length > 1) {
                return true;
            }
        }
        return false;
    }
    acyclic() {
        return !this.cyclic();
    }
    /**
     * Uses Tarjan's algorithm with Nuutila's modifications. Nonrecursive.
     */
    strongly_connected_components() {
        const preorder = new Map();
        const lowlink = new Map();
        const scc_found = new Map();
        const scc_queue = [];
        const result = [];
        let i = 0; // preorder counter
        for (const source of this.vertices) {
            if (!scc_found.has(source)) {
                const queue = [source];
                while (queue.length > 0) {
                    const v = queue[queue.length - 1];
                    if (!preorder.has(v)) {
                        i += 1;
                        preorder.set(v, i);
                    }
                    let done = true;
                    const v_neighbors = this.neighbors(v);
                    for (const w of v_neighbors) {
                        if (!preorder.has(w)) {
                            queue.push(w);
                            done = false;
                            break;
                        }
                    }
                    if (done) {
                        lowlink.set(v, preorder.get(v));
                        for (const w of v_neighbors) {
                            if (!scc_found.has(w)) {
                                if (preorder.get(w) > preorder.get(v)) {
                                    lowlink.set(v, Math.min(lowlink.get(v), lowlink.get(w)));
                                }
                                else {
                                    lowlink.set(v, Math.min(lowlink.get(v), preorder.get(w)));
                                }
                            }
                        }
                        queue.pop();
                        if (lowlink.get(v) === preorder.get(v)) {
                            scc_found.set(v, true);
                            const scc = [];
                            scc.push(v);
                            while (scc_queue.length > 0 && preorder.get(scc_queue[scc_queue.length - 1]) > preorder.get(v)) {
                                const k = scc_queue.pop();
                                scc_found.set(k, true);
                                scc.push(k);
                            }
                            result.push(scc);
                        }
                        else {
                            scc_queue.push(v);
                        }
                    }
                }
            }
        }
        return result;
    }
    /**
     * Uses Johnson's cycle finding algorithm.
     */
    simple_cycles() {
        const unblock = (thisnode, blocked, B) => {
            const stack = [thisnode];
            while (stack.length > 0) {
                const node = stack.pop();
                if (blocked.has(node)) {
                    blocked.delete(node);
                    if (B.has(node)) {
                        for (const n of B.get(node)) {
                            if (stack.indexOf(n) === -1) {
                                stack.push(n);
                            }
                        }
                        B.get(node).clear();
                    }
                }
            }
        };
        const result = [];
        const subG = this.clone();
        const sccs = [];
        for (const scc of this.strongly_connected_components()) {
            sccs.push(scc);
        }
        while (sccs.length > 0) {
            const scc = sccs.pop();
            const start_node = scc.pop();
            const path = [start_node];
            const blocked = new Set();
            const closed = new Set();
            blocked.add(start_node);
            const B = new Map();
            const stack = [[start_node, subG.neighbors(start_node)]];
            while (stack.length > 0) {
                const [thisnode, neighbors] = stack[stack.length - 1];
                if (neighbors.length > 0) {
                    const next_node = neighbors.pop();
                    if (next_node === start_node) {
                        const result_path = [];
                        for (const v of path) {
                            result_path.push(v);
                        }
                        result.push(result_path);
                        for (const v of path) {
                            closed.add(v);
                        }
                    }
                    else if (!blocked.has(next_node)) {
                        path.push(next_node);
                        stack.push([next_node, subG.neighbors(next_node)]);
                        closed.delete(next_node);
                        blocked.add(next_node);
                        continue;
                    }
                }
                if (neighbors.length === 0) {
                    if (closed.has(thisnode)) {
                        unblock(thisnode, blocked, B);
                    }
                    else {
                        for (const neighbor of subG.neighbors(thisnode)) {
                            if (!B.has(neighbor)) {
                                B.set(neighbor, new Set());
                            }
                            B.get(neighbor).add(thisnode);
                        }
                    }
                    stack.pop();
                    path.pop();
                }
            }
            subG.remove_vertex(start_node);
            const H = subG.subgraph(scc);
            const H_sccs = H.strongly_connected_components();
            for (const H_scc of H_sccs) {
                sccs.push(H_scc);
            }
        }
        // remove duplicates
        const eqSet = (as, bs) => {
            if (as.size !== bs.size) {
                return false;
            }
            for (const a of as) {
                if (!bs.has(a)) {
                    return false;
                }
            }
            return true;
        };
        const deduplicate = (cycles) => {
            const uniques = [];
            for (const array of cycles) {
                const set = new Set(array);
                let add = true;
                for (const unique of uniques) {
                    if (eqSet(set, unique)) {
                        add = false;
                    }
                }
                if (add) {
                    uniques.push(set);
                }
            }
            return uniques.map((set) => {
                const array = [];
                for (const value of set) {
                    array.push(value);
                }
                return array;
            });
        };
        return deduplicate(result);
    }
    subgraph(vertices) {
        const subgraph = new DirectedGraph();
        subgraph.add_vertices(...vertices);
        for (const v of this.vertices) {
            if (vertices.indexOf(v) !== -1) {
                const neighbors = this.neighbors(v);
                for (const u of neighbors) {
                    if (vertices.indexOf(u) !== -1) {
                        subgraph.add_edge(v, u);
                    }
                }
            }
        }
        return subgraph;
    }
    remove_edge(edge) {
        const neighbors = this._edges.get(edge[0]);
        neighbors.splice(neighbors.indexOf(edge[1]), 1);
    }
}
exports.DirectedGraph = DirectedGraph;
