"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
const component_manager_1 = require("./component_manager");
const detector_1 = require("./engines/detector");
const entity_creator_1 = require("./entity_creator");
const entity_manager_1 = require("./entity_manager");
const detector_without_component_types_1 = require("./errors/detector_without_component_types");
const engine_cycle_1 = require("./errors/engine_cycle");
const engine_mutation_conflict_1 = require("./errors/engine_mutation_conflict");
const directed_graph_1 = require("./graphs/directed_graph");
const message_manager_1 = require("./message_manager");
const render_manager_1 = require("./render_manager");
const type_guards_1 = require("./type_guards");
const world_1 = require("./world");
/**
 * A WorldBuilder is used to build a World out of Engines and Renderers.
 *
 * The WorldBuilder enforces certain rules about Engine structure.
 * It is forbidden to have messages create cycles between Engines,
 * and no Component may be mutated by more than one Engine.
 *
 * The WorldBuilder uses Engines and their Message read/emit information
 * to determine a valid ordering of the Engines,
 * which is given to the World.
 */
class WorldBuilder {
    constructor() {
        this.engines = new encompass_gc_optimized_collections_1.GCOptimizedList();
        this.renderers = new encompass_gc_optimized_collections_1.GCOptimizedList();
        this.engine_graph = new directed_graph_1.DirectedGraph();
        this.message_type_to_emitters = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        this.message_type_to_readers = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        const entities = new encompass_gc_optimized_collections_1.GCOptimizedList();
        const id_to_entity = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        const entities_with_added_components = new encompass_gc_optimized_collections_1.GCOptimizedList();
        const entities_with_removed_components = new encompass_gc_optimized_collections_1.GCOptimizedList();
        const entities_marked_for_destroy = new encompass_gc_optimized_collections_1.GCOptimizedList();
        this.render_manager = new render_manager_1.RenderManager(id_to_entity);
        this.component_manager = new component_manager_1.ComponentManager();
        this.entity_creator = new entity_creator_1.EntityCreator(entities, id_to_entity, entities_with_added_components, entities_with_removed_components, entities_marked_for_destroy, this.component_manager, this.render_manager.draw_layer_manager);
        this.entity_manager = new entity_manager_1.EntityManager(entities, id_to_entity, entities_with_added_components, entities_with_removed_components, entities_marked_for_destroy);
        this.message_manager = new message_manager_1.MessageManager();
        this.world = new world_1.World(this.component_manager, this.entity_creator, this.entity_manager, this.message_manager, this.render_manager);
    }
    /**
     * Instantiates a new Entity.
     */
    create_entity() {
        return this.entity_creator.create_entity();
    }
    /**
     * Creates a new message.
     * @param MessageType
     */
    emit_message(MessageType) {
        return this.message_manager.create_message(MessageType);
    }
    /**
     * Creates a new component message to be consumed
     * immediately upon the next World update.
     *
     * @param ComponentMessageType
     * @param component
     */
    emit_component_message(ComponentMessageType, component) {
        return this.message_manager.create_component_message(ComponentMessageType, component);
    }
    /**
     * Creates a new EntityMessage to be consumed
     * immediately upon the next World update.
     *
     * @param EntityMessageType
     * @param entity
     */
    emit_entity_message(EntityMessageType, entity) {
        return this.message_manager.create_entity_message(EntityMessageType, entity);
    }
    /**
     * Adds an engine to the WorldBuilder.
     *
     * @param EngineType
     */
    add_engine(EngineType) {
        const engine = new EngineType(this.entity_manager, this.entity_creator, this.component_manager, this.message_manager);
        this.engines.add(engine);
        this.engine_graph.add_vertex(engine);
        if (engine instanceof detector_1.Detector) {
            this.entity_manager.register_entity_tracker(engine);
            if (!engine.component_types) {
                const detect_error = new detector_without_component_types_1.DetectorWithoutComponentTypesError(this.constructor.name);
                throw detect_error.toString();
            }
        }
        for (const emit_message_type of engine.emit_message_types.values()) {
            if (!this.message_type_to_emitters.has(emit_message_type)) {
                this.message_type_to_emitters.set(emit_message_type, new encompass_gc_optimized_collections_1.GCOptimizedSet());
            }
            this.message_type_to_emitters.get(emit_message_type).add(engine);
            if (this.message_type_to_readers.has(emit_message_type)) {
                for (const reader of this.message_type_to_readers.get(emit_message_type).entries()) {
                    this.engine_graph.add_edge(engine, reader);
                }
            }
        }
        for (const read_message_type of engine.read_message_types.values()) {
            if (!this.message_type_to_readers.has(read_message_type)) {
                this.message_type_to_readers.set(read_message_type, new encompass_gc_optimized_collections_1.GCOptimizedSet());
            }
            this.message_type_to_readers.get(read_message_type).add(engine);
            if (this.message_type_to_emitters.has(read_message_type)) {
                for (const emitter of this.message_type_to_emitters.get(read_message_type).entries()) {
                    this.engine_graph.add_edge(emitter, engine);
                }
            }
        }
        return engine;
    }
    /**
     * Adds a new Renderer engine to the WorldBuilder.
     *
     * @param RendererType
     */
    add_renderer(RendererType) {
        const renderer = new RendererType(this.entity_manager, this.component_manager);
        this.renderers.add(renderer);
        if (type_guards_1.TypeGuard.renderer_is_entity_renderer(renderer)) {
            this.render_manager.register_entity_renderer(renderer);
            this.entity_manager.register_drawable_entity_tracker(renderer);
        }
        else if (type_guards_1.TypeGuard.renderer_is_general_renderer(renderer)) {
            this.render_manager.draw_layer_manager.register_object_with_layer(renderer, renderer.layer);
        }
        return renderer;
    }
    /**
     * Call this when you are done adding Engines and Renderers.
     *
     * @throws {EngineCycleError} When the Engine emit/reads produce a cycle between Engines.
     * @throws {EngineMutationConflictError} When multiple Engines mutate the same Component type.
     *
     * @returns {World} An instantiated World with a valid Engine ordering.
     */
    build() {
        if (this.engine_graph.cyclic()) {
            const engine_cycle_error = new engine_cycle_1.EngineCycleError(this.engine_graph.simple_cycles());
            throw engine_cycle_error.toString();
        }
        const mutated_component_types = new Set();
        const duplicate_mutations = [];
        const component_to_engines_map = new Map();
        for (const engine of this.engines.values()) {
            for (const mutated_component_type of engine.mutate_component_types.values()) {
                if (mutated_component_types.has(mutated_component_type)) {
                    duplicate_mutations.push(mutated_component_type);
                }
                else {
                    mutated_component_types.add(mutated_component_type);
                }
                if (!component_to_engines_map.has(mutated_component_type)) {
                    component_to_engines_map.set(mutated_component_type, []);
                }
                component_to_engines_map.get(mutated_component_type).push(engine);
            }
        }
        if (duplicate_mutations.length > 0) {
            const engine_mutation_conflict_error = new engine_mutation_conflict_1.EngineMutationConflictError(duplicate_mutations, component_to_engines_map);
            throw engine_mutation_conflict_error.toString();
        }
        const gc_optimized_engine_order = new encompass_gc_optimized_collections_1.GCOptimizedList();
        for (const engine of this.engine_graph.topological_sort()) {
            gc_optimized_engine_order.add(engine);
        }
        this.world.engine_order = gc_optimized_engine_order;
        this.entity_manager.check_entities_with_added_components();
        this.entity_manager.check_entities_with_removed_components();
        this.component_manager.activate_components();
        this.component_manager.deactivate_components();
        this.component_manager.remove_components();
        return this.world;
    }
}
exports.WorldBuilder = WorldBuilder;
