import { Component } from "./component";
import { ComponentManager } from "./component_manager";
import { Engine } from "./engine";
import { Entity } from "./entity";
import { EntityCreator } from "./entity_creator";
import { EntityManager } from "./entity_manager";
import { Message } from "./message";
import { MessageManager } from "./message_manager";
import { Renderer } from "./renderer";
import { ComponentMessage } from "./types/component_message";
import { EntityMessage } from "./types/entity_message";
import { Type } from "./types/generic_type";
import { World } from "./world";
/**
 * A WorldBuilder is used to build a World out of Engines and Renderers.
 *
 * The WorldBuilder enforces certain rules about Engine structure.
 * It is forbidden to have messages create cycles between Engines,
 * and no Component may be mutated by more than one Engine.
 *
 * The WorldBuilder uses Engines and their Message read/emit information
 * to determine a valid ordering of the Engines,
 * which is given to the World.
 */
export declare class WorldBuilder {
    private engines;
    private renderers;
    private engine_graph;
    private message_type_to_emitters;
    private message_type_to_readers;
    private component_manager;
    private entity_creator;
    private entity_manager;
    private message_manager;
    private render_manager;
    private world;
    constructor();
    /**
     * Instantiates a new Entity.
     */
    create_entity(): Entity;
    /**
     * Creates a new message.
     * @param MessageType
     */
    emit_message<TMessage extends Message>(MessageType: Type<TMessage>): TMessage;
    /**
     * Creates a new component message to be consumed
     * immediately upon the next World update.
     *
     * @param ComponentMessageType
     * @param component
     */
    emit_component_message<TMessage extends ComponentMessage>(ComponentMessageType: Type<TMessage>, component: Component): TMessage;
    /**
     * Creates a new EntityMessage to be consumed
     * immediately upon the next World update.
     *
     * @param EntityMessageType
     * @param entity
     */
    emit_entity_message<TMessage extends EntityMessage>(EntityMessageType: Type<TMessage>, entity: Entity): TMessage;
    /**
     * Adds an engine to the WorldBuilder.
     *
     * @param EngineType
     */
    add_engine<TEngine extends Engine>(EngineType: new (entity_manager: EntityManager, entity_creator: EntityCreator, component_manager: ComponentManager, message_manager: MessageManager) => TEngine): TEngine;
    /**
     * Adds a new Renderer engine to the WorldBuilder.
     *
     * @param RendererType
     */
    add_renderer<TRenderer extends Renderer>(RendererType: new (entity_manager: EntityManager, component_manager: ComponentManager) => TRenderer): TRenderer;
    /**
     * Call this when you are done adding Engines and Renderers.
     *
     * @throws {EngineCycleError} When the Engine emit/reads produce a cycle between Engines.
     * @throws {EngineMutationConflictError} When multiple Engines mutate the same Component type.
     *
     * @returns {World} An instantiated World with a valid Engine ordering.
     */
    build(): World;
}
